<?php

namespace Tvip\CoreBundle\Controller\ProviderApi;

use Doctrine\Common\Inflector\Inflector;
use Doctrine\ORM\PersistentCollection;
use JMS\Serializer\EventDispatcher\PreDeserializeEvent;
use JMS\Serializer\GenericSerializationVisitor;
use JMS\Serializer\Metadata\PropertyMetadata;
use Symfony\Bridge\Doctrine\ManagerRegistry;
use JMS\Serializer\Naming\PropertyNamingStrategyInterface;
use JMS\Serializer\Serializer;
use Doctrine\ORM\EntityManager;
use JMS\Serializer\EventDispatcher\EventSubscriberInterface;
use JMS\Serializer\EventDispatcher\ObjectEvent;

class ProviderApiSerializer implements EventSubscriberInterface
{
    /**
     * @var ManagerRegistry
     */
    protected $managerRegistry;

    /**
     * @var PropertyNamingStrategyInterface
     */
    protected $namingStrategy;

    /**
     * @var Serializer
     */
    protected $serializer;


    public function __construct(ManagerRegistry $managerRegistry, PropertyNamingStrategyInterface $namingStrategy, Serializer $serializer){
        $this->managerRegistry=$managerRegistry;
        $this->namingStrategy=$namingStrategy;
        $this->serializer = $serializer;
    }

    static public function getSubscribedEvents()
    {
        return array(
            array('event' => 'serializer.post_serialize', 'method' => 'onPostSerialize'),
            array('event' => 'serializer.pre_deserialize', 'method' => 'onPreDeserialize'),
        );
    }


    public function onPostSerialize(ObjectEvent $event)
    {
        $object = $event->getObject();
        $type = $event->getType();
        $context = $event->getContext();

        // Locate possible ObjectManager

        /** @var EntityManager $em */
        $em = $this->managerRegistry->getManagerForClass($type['name']); //based on JMS\Serializer\Construction\DoctrineObjectConstructor class

        if (!$em) {
            // ignore if not entity
            return;
        }

        $visitor=$event->getVisitor();

        $entityMetaData=$em->getClassMetadata($type['name']);

        $classMetaData=$context->getMetadataFactory()->getMetadataForClass($type['name']);

        foreach ($entityMetaData->getAssociationMappings() as $assocName => $assoc) {
            if ($entityMetaData->isSingleValuedAssociation($assocName)) {

                /** @var PropertyMetadata $propertyMetaData */
                $propertyMetaData=$classMetaData->propertyMetadata[$assocName];
                $exclusionStrategy = $context->getExclusionStrategy();

                if($exclusionStrategy->shouldSkipProperty($propertyMetaData,$context)==false){

                    $targetEntityMetaData=$em->getClassMetadata($assoc['targetEntity']);
                    $target=NULL;
                    $target=$propertyMetaData->getValue($object);

                    $identifiers=array();
                    if ($target) {
                        // Association is set - let's use keys
                        $identifiers = $targetEntityMetaData->getIdentifierValues($target);
                    } elseif ($context->shouldSerializeNull()) {
                        // fill with NULL if need serialize null
                        foreach ($targetEntityMetaData->getIdentifierFieldNames() as $idKey){
                            $identifiers[$idKey]=NULL;
                        }
                    }

                    foreach ($identifiers as $idKey=>$idValue) {
                        $idFieldName = $this->translatePropertyName($assocName);
                        $idFieldValue = $idValue;

                        if ($visitor instanceof GenericSerializationVisitor) {
                            $visitor->addData($idFieldName, $idFieldValue);
                        } else {
                            throw new \Exception("Visitor of class " . get_class($visitor) . " is not supported");
                        }
                    }
                }
            }
        }
    }

    public function onPreDeserialize(PreDeserializeEvent $event){
        $em = $this->managerRegistry->getManagerForClass($event->getType()['name']);
        if(!$em){
            return;
        }

        $entityMetaData = $em->getClassMetadata($event->getType()['name']);
        $properties = $event->getData();
        $originalProperties = array();

        $idProperties = $entityMetaData->getIdentifier();

        if(count($idProperties)!=1) throw new ProviderApiException('Entity with multiply id is not supported');

        $idProperty = $idProperties[0];

        //PATCH entity
        if(isset($properties[$idProperty])){
            $original = $em->getRepository($event->getType()['name'])->find($properties[$idProperty]);
            if(method_exists($original,'__load')){
                $original->__load();
            }else{
                $reflection = $em->getClassMetadata($event->getType()['name'])->getReflectionClass();
                foreach($reflection->getProperties() as $reflectionProperty) {
                    $reflectionProperty->setAccessible(true);
                    if(!$reflectionProperty->getValue($original) instanceof PersistentCollection){

                        $propertyName = $this->translatePropertyName($reflectionProperty->getName());
                        $propertyValue = $reflectionProperty->getValue($original);

                        if($entityMetaData->hasAssociation($reflectionProperty->getName())){
                            //TODO: use reflections to get id
                            if(method_exists($reflectionProperty->getValue($original),'getId'))  $propertyValue = array('id'=>$reflectionProperty->getValue($original)->getId());
                        }

                         if(is_object($propertyValue)){
                            if(method_exists($propertyValue,'__toString')) $propertyValue = $propertyValue->__toString();
                             
                             //TODO: fetch format from Serializer/Type

                            if($propertyValue instanceof \DateTime) $propertyValue = $propertyValue->format('Y-m-d H:i:s');
                        }

                        $originalProperties[$propertyName] = $propertyValue;
                    }
                    $reflectionProperty->setAccessible(false);
                }

                foreach($properties as $propertyName=>$propertyValue){
                    if($entityMetaData->hasAssociation(Inflector::camelize($propertyName))){
                        $properties[$propertyName] = $propertyValue ? array('id'=>$propertyValue) :null;
                    }

                }

                $properties = array_merge($originalProperties,$properties);
                $event->setData($properties);
            }

        }
        else{
            foreach($properties as $propertyName=>$propertyValue){
                if($entityMetaData->hasAssociation(Inflector::camelize($propertyName))){
                    $properties[$propertyName] = $propertyValue ? array('id'=>$propertyValue) :null;
                }
            }
            $event->setData($properties);
        }

    }

    protected function translatePropertyName($name){
        return strtolower(preg_replace('/[A-Z]/', '_'.'\\0', $name));
    }
}